# py/pyext - python script objects for PD and MaxMSP
#
# Copyright (c) 2002-2007 Thomas Grill (gr@grrrr.org)
# For information on usage and redistribution, and for a DISCLAIMER OF ALL
# WARRANTIES, see the file, "license.txt," in this distribution.  
#

"""This is an example script for the py/pyext object's basic functionality.

pyext Usage:
- Import pyext

- Inherit your class from pyext._class

- Specfiy the number of inlets and outlets:
	Use the class members (variables) _inlets and _outlets
	If not given they default to 1
	You can also use class methods with the same names to return the respective number

- Constructors/Destructors
	You can specify an __init__ constructor and/or an __del__ destructor.
	The constructor will be called with the object's arguments

	e.g. if your PD or MaxMSP object looks like
	[pyext script class arg1 arg2 arg3]

	then the __init__(self,*args) function will be called with a tuple argument
	args = (arg1,arg2,arg3) 
	With this syntax, you will have to give at least one argument.
	By defining the constructor as __init__(self,*args) you can also initialize 
	the class without arguments.

- Methods called by pyext
	The general format is 'tag_inlet(self,arg)' resp. 'tag_inlet(self,*args)':
		tag is the PD or MaxMSP message header.. either bang, float, list etc.
		inlet is the inlet (starting from 1) from which messages are received.
		args is a tuple which corresponds to the content of the message. args can be omitted.

	The inlet index can be omitted. The method name then has the format 'tag_(self,inlet,args)'.
	Here, the inlet index is a additional parameter to the method

	You can also set up methods which react on any message. These have the special forms
		_anything_inlet(self,*args)
	or
		_anything_(self,inlet,*args) 

	Please see below for examples.

	Any return values are ignored - use _outlet (see below).

	Generally, you should avoid method_, method_xx forms for your non-pyext class methods.
	Identifiers (variables and functions) with leading underscores are reserved for pyext.

- Send messages to outlets:
	Use the inherited _outlet method.
	You can either use the form
		self._outlet(outlet,arg1,arg2,arg3,arg4) ... where all args are atoms (no sequence types!)
	or
		self._outlet(outlet,arg) ... where arg is a sequence containing only atoms
		
    Do not use _outlet inside __init__, since the outlets have not been created at that time.

- Use pyext functions and methods: 
	See the __doc__ strings of the pyext module and the pyext._class base class.

"""

try:
	import pyext
except:
	print "ERROR: This script must be loaded by the PD/Max pyext external"

#################################################################

import rtmidi
from rtmidi.midiutil import open_midiport


class japp(object):


        def say_close(self, what="close"):

                self.midiin.close_port()
                del self.midiin
                print " %s closed" % (self.port_name)

        def __init__(self,xyz,port="0"):
		self.xyz = xyz
		self.port = port
	
		midiin, self.port_name = open_midiport(self.port)
                print " %s opened" % (self.port_name)

                self.midiin = midiin

                midiin.set_callback(self.MidiInputHandler(self.xyz))
                midiin.ignore_types(sysex=False, timing=False, active_sense=True)

        class MidiInputHandler(object):
                def __init__(self, pxtcl):
                        self.pxtcl = pxtcl

                def __call__(self, event, data=None):
                        message, deltatime = event
                        
                        if self.pxtcl.listmode != "0":
                                self.pxtcl._outlet(1,message)
                        else:
                                for x in message:
                                        self.pxtcl._outlet(1,x)
                                        
                                        

                                               

class midiinput(pyext._class):
	_inlets = 1
	_outlets = 1

	
        def __init__(self,*args):
                self.openend = False
                self.listmode = "0"
                
                if len(args) >> 0:
                        self.portnr = args[0]
                else:
                        self.portnr = "0"
                        
                print "Midiportnumber: %s" % (self.portnr)
                
	def open_1(self):
                if self.openend == False:
        
                        self.app = japp(self, self.portnr )
                        self.openend = True
                else:
                        print "it's open!"


	def close_1(self):
                if self.openend == True:
                        self.app.say_close()
                        self.openend = False

                else:
                        print "it's closed!"

        def listmode_1(self, i):
                self.listmode = str(i)


